%This code estimates vector autoregressive (multivariate) and
%autoregressive models with the purpose of forecasting core inflation in
%Chile. The data consists of the nominal exchange rate, core inflation,
%core goods inflation, and core services inflation. The sample covers the
%period 2002:M2-2020:M2. The data is expressed as log differences times
%100. The code computes a VAR(2) model (2 lags) for the exchange rate and
%inflation (bi-variate), and an AR(2) model (2 lags) for inflation.
%For analyzing forecasting performance, the code computes recursive
%forecasts 6 months ahead, starting with the sample 2002:M2-2015:M2, and
%ending with the sample 2002:M2-2019:M8, i.e., 55 periods of recursive
%forecasts. The code computes the root mean squared error (RMSE) by horizon
%(1 month ahead, months ahead, etc.), and the RMSE for each recursive
%iteration (the RMSE for the forecast using the sample 2002:M2-2015:M2, the
%RMSE for the forecast using the sample 2002:M2-2015:M3, etc.).
%If you want to alter the specification (number of variables, number of
%lags, etc.), you have to read and adjust the code line by line. It's not
%written so as to be modified by setting a few options.
%The notation is based on the handbook and accompanying
%code by Andrew Blake and Haroon Mumtaz: Applied Bayesian Econometrics for
%Central Bankers. The code uses functions provided by the authors.
%Author: Juan Guerra-Salas
%October 2020

close all
clear
clc
addpath('functions'); %functions written by Blake and Mumtaz
addpath('data');
data=xlsread('\data\data_exrate_inf_Chile.xlsx'); %exch. rate, core inf., core inf. serv., core inf. goods. 2002:M2-2020:M2 

%Plot variables
tt=2002+1/12:1/12:2020+1/12;
figure(1)
subplot(2,2,1)
plot(tt,data(:,1)); grid on
title('Nom. exch. rate');
axis tight
subplot(2,2,2)
plot(tt,data(:,2)); grid on
title('Core inflation');
axis tight
subplot(2,2,3)
plot(tt,data(:,3)); grid on
title('Core inf. services');
axis tight
subplot(2,2,4)
plot(tt,data(:,4)); grid on
title('Core inf. goods');
axis tight

%VAR(2) model
N=2; %size(data,2); %number of endogenous variables
L=2; %number of lags in the VAR
Y=data(:,[1 4]); %matrix of endogenous variables
recursive_periods=size(Y(1:end-6,:),1)-size(Y(1:end-12*5,:),1)+1; %Start recursive estimation for sample 2002:M2-2015:M2. End with sample 2002:M2-2019:M9.
er_fcasts=NaN(size(data,1),recursive_periods); %matrix that will contain recursive forecasts of exch. rate
inf_fcasts_var=NaN(size(data,1),recursive_periods); %matrix that will contain recursive forecasts of inflation
rmse_var=zeros(1,recursive_periods); %matrix that will contain root mean squared error of inflation forecasts
se_var=zeros(6,recursive_periods); %matrix that will contain squared errors of inflation forecasts
for j=1:recursive_periods 
    Y_iter=Y(1:end-recursive_periods+j-6,:); %matrix of endogenous variables for each iteration
    X=[ones(size(Y_iter,1),1) lag0(Y_iter,1) lag0(Y_iter,2) ]; %matrix of regressors: constant and 1st and 2nd lags of variables
    Y_iter=Y_iter(L+1:end,:); %adjust sample for observations that are lost due to lags
    X=X(L+1:end,:);
    T=rows(X); %number of observations (adjusted)
    B=inv(X'*X)*(X'*Y_iter); %estimation of parameters by OLS
    e=Y_iter-X*B; %residuals
    Sigma=(e'*e)/(T-(L*N+1)); %var-cov matrix of residuals corrected for degrees of freedom (L*N+1 is the # of param. per eqn. when including a cons)
    yhat=zeros(L+6,2); %vector that will contain forecast
    yhat(1:L,:)=Y_iter(end-L+1:end,:); %starting values for forecast (model uses L lags, so we need the last L observations)
    for m=L+1:L+6 %this loop computes the forecast by simulating the model
        yhat(m,:)=[1 yhat(m-1,:) yhat(m-2,:) ]*B;
    end
    er_fcasts(j-1+size(Y(1:end-12*5,:),1):j+size(Y(1:end-12*5,:),1)+6-1,j)=yhat(L:end,1); %save forecast of exch. rate
    inf_fcasts_var(j-1+size(Y(1:end-12*5,:),1):j+size(Y(1:end-12*5,:),1)+6-1,j)=yhat(L:end,2); %save forecast of inflation
    rmse_var(j)=sqrt(sum((yhat(L+1:end,2)-Y(end-recursive_periods+j-6+1:end-recursive_periods+j,2)).^2)/6); %compute root mean squared error of inflation forecasts
    se_var(:,j)=(yhat(L+1:end,2)-Y(end-recursive_periods+j-6+1:end-recursive_periods+j,2)).^2; %compute root mean squared error of inflation forecasts
end

%AR(2) model
N=1; %size(data,2); %number of endogenous variables
L=2; %number of lags in the VAR
Y2=data(:,[4]); %matrix of endogenous variables
recursive_periods=size(Y2(1:end-6,:),1)-size(Y2(1:end-12*5,:),1)+1; %Start recursive estimation for sample 2002:M2-2015:M2. End with sample 2002:M2-2019:M9.
inf_fcasts_ar=NaN(size(data,1),recursive_periods); %matrix that will contain recursive forecasts of inflation
rmse_ar=zeros(1,recursive_periods); %matrix that will contain root mean squared error of inflation forecasts
se_ar=zeros(6,recursive_periods); %matrix that will contain squared errors of inflation forecasts
for j=1:recursive_periods 
    Y2_iter=Y2(1:end-recursive_periods+j-6,:); %matrix of endogenous variables for each iteration
    X2=[ones(size(Y2_iter,1),1) lag0(Y2_iter,1) lag0(Y2_iter,2) ]; %matrix of regressors: constant and 1st and 2nd lags of variables
    Y2_iter=Y2_iter(L+1:end,:); %adjust sample for observations that are lost due to lags
    X2=X2(L+1:end,:);
    T=rows(X2); %number of observations (adjusted)
    B2=inv(X2'*X2)*(X2'*Y2_iter); %estimation of parameters by OLS
    e2=Y2_iter-X2*B2; %residuals
    yhat2=zeros(L+6,1); %vector that will contain forecast
    yhat2(1:L)=Y2_iter(end-L+1:end); %starting values for forecast (model uses L lags, so we need the last L observations)
    for m=L+1:L+6 %this loop computes the forecast by simulating the model
        yhat2(m)=[1 yhat2(m-1) yhat2(m-2) ]*B2;
    end
    inf_fcasts_ar(j-1+size(Y2(1:end-12*5),1):j+size(Y2(1:end-12*5),1)+6-1,j)=yhat2(L:end); %save forecast of inflation
    rmse_ar(j)=sqrt(sum((yhat2(L+1:end)-Y2(end-recursive_periods+j-6+1:end-recursive_periods+j)).^2)/6); %compute root mean squared error of inflation forecasts
    se_ar(:,j)=(yhat2(L+1:end)-Y2(end-recursive_periods+j-6+1:end-recursive_periods+j)).^2; %compute root mean squared error of inflation forecasts
end

%Plot data and recursive forecasts of inflation
figure(2)
subplot(2,1,1)
plot(tt,Y(:,2),'linewidth',3); grid on
hold on
for i=1:size(inf_fcasts_var,2)
    plot(tt,inf_fcasts_var(:,i),'color','red');
    hold on
end
title('Recursive fcasts: inflation (VAR2)');
axis tight
xlim([2013 2020+1/12]);
subplot(2,1,2)
plot(tt,Y2,'linewidth',3); grid on
hold on
for i=1:size(er_fcasts,2)
    plot(tt,inf_fcasts_ar(:,i),'color','red');
    hold on
end
title('Recursive fcasts: inflation (AR2)');
axis tight
xlim([2013 2020+1/12]);

%Forecasting performance
rmse_var_horizon=sqrt(mean(se_var,2)); %compute mean squared error across horizons
rmse_ar_horizon=sqrt(mean(se_ar,2));
figure(3)
plot(rmse_var_horizon);
hold on
plot(rmse_ar_horizon);
title('Root mean squared error by horizon');
axis tight
legend('VAR model','AR model');

figure(4)
plot(rmse_var);
hold on
plot(rmse_ar);
title('Root mean squared error per iteration');
axis tight
legend('VAR model','AR model');